@@ -35,17 +35,17 @@ robmon <- function(z, x, useCluster, nbrNodes, initC, clusterString,
     {
         x$FRAN <- getFromNamespace(x$FRANname, pos=grep("RSiena", search())[1])
     }
-    z <- x$FRAN(z, x, INIT=TRUE, ...)
+    z <- x$FRAN(z, x, INIT=TRUE, initC=FALSE, ...)
     ##
     ##if conditional, FRAN changes z$theta etc
     #######################################################
     if (useCluster)
     {
-        if (!is.null(x$FRANname) && x$FRANname != "simstats0c")
-        {
-            stop("Multiple processors only for simstats0c at present")
-        }
-        if (!clusterIter && nbrNodes >= z$f$observations)
+       # if (!is.null(x$FRANname) && x$FRANname != "simstats0c")
+       # {
+       #     stop("Multiple processors only for simstats0c at present")
+       # }
+        if (!clusterIter && nbrNodes >= z$observations)
         {
             stop("Not enough observations to use the nodes")
         }
