@@ -12,9 +12,9 @@
 
 ##@siena07 siena07
 siena07<- function(x, batch = FALSE, verbose = FALSE, silent=FALSE,
-                   useCluster = FALSE, nbrNodes = 2, initC=FALSE,
+                   useCluster = FALSE, nbrNodes = 2, initC=TRUE,
                    clusterString=rep("localhost", nbrNodes), tt=NULL,
-                   parallelTesting=FALSE, clusterIter=TRUE, ...)
+                   parallelTesting=FALSE, clusterIter=!x$maxlike, ...)
 {
     exitfn <- function()
     {
@@ -39,7 +39,7 @@ siena07<- function(x, batch = FALSE, verbose = FALSE, silent=FALSE,
         {
             stop("cannot parallel test with multiple processes")
         }
-        require(snow)
+        require(snow, warn.conflicts=FALSE)
         require(rlecuyer)
         if (clusterIter)
         {
