@@ -21,16 +21,16 @@ namespace siena
  * Average alter effect defined as the product of the ego with the average
  * of its neighbors (with respect to a certain network).
  */
-	class AverageAlterEffect : public NetworkDependentBehaviorEffect
+class AverageAlterEffect : public NetworkDependentBehaviorEffect
 {
 public:
 	AverageAlterEffect(const EffectInfo * pEffectInfo);
 
 	virtual double calculateChangeContribution(int actor,
-		int difference) const;
-	virtual double evaluationStatistic(double * currentValues) const;
+		int difference);
 	virtual double endowmentStatistic(const int * difference,
-		double * currentValues) const;
+		double * currentValues);
+	virtual double egoStatistic(int ego, double * currentValues);
 };
 
 }
