@@ -1,10 +1,10 @@
 /******************************************************************************
  * SIENA: Simulation Investigation for Empirical Network Analysis
- * 
+ *
  * Web: http://www.stats.ox.ac.uk/~snijders/siena/
- * 
+ *
  * File: EffectFactory.h
- * 
+ *
  * Description: This file contains the definition of the
  * EffectFactory class.
  *****************************************************************************/
@@ -19,6 +19,7 @@ namespace siena
 // Section: Forward declarations
 // ----------------------------------------------------------------------------
 
+class Data;
 class Effect;
 class EffectInfo;
 
@@ -29,12 +30,17 @@ class EffectInfo;
 
 /**
  * Produces concrete effects of the Effect class hierarchy from generic
- * effect descriptor objects of class EffectInfo. 
+ * effect descriptor objects of class EffectInfo.
  */
 class EffectFactory
 {
 public:
+	EffectFactory(const Data * pData);
+
 	Effect * createEffect(const EffectInfo * pEffectInfo) const;
+
+private:
+	const Data * lpData;
 };
 
 }
