@@ -19,6 +19,7 @@ enum Aspect {NETWORK, BEHAVIOR};
 
 class Chain;
 class MiniStep;
+class Option;
 
 
 // ----------------------------------------------------------------------------
@@ -31,6 +32,7 @@ public:
 	MLSimulation(Data * pData, Model * pModel);
 	virtual ~MLSimulation();
 
+    void initialize(int period);
 	void connect(int period);
     void updateProbabilities(const Chain * pChain,
     	MiniStep * pFirstMiniStep,
@@ -43,7 +45,7 @@ public:
 
 	int acceptances(int stepType) const;
 	int rejections(int stepType) const;
-	const Chain * pChain() const;
+	Chain * pChain() const;
 	void pChain(Chain * pChain);
 	void pChainProbabilities(Chain * pChain, int period);
 
@@ -54,6 +56,8 @@ public:
 	bool permute(int c0);
 	bool insertPermute(int c0);
 	bool deletePermute(int c0);
+	bool insertMissing();
+	bool deleteMissing();
 	double proposalProbability() const;
 	bool missingData() const;
 	Aspect aspect() const;
@@ -61,27 +65,6 @@ public:
 	void simpleRates(bool flag);
 	bool simpleRates() const;
 
-	void insertDiagonalProbability(double probability);
-	double insertDiagonalProbability() const;
-
-	void cancelDiagonalProbability(double probability);
-	double cancelDiagonalProbability() const;
-
-	void permuteProbability(double probability);
-	double permuteProbability() const;
-
-	void insertPermuteProbability(double probability);
-	double insertPermuteProbability() const;
-
-	void deletePermuteProbability(double probability);
-	double deletePermuteProbability() const;
-
-	void insertRandomMissingProbability(double probability);
-	double insertRandomMissingProbability() const;
-
-	void deleteRandomMissingProbability(double probability);
-	double deleteRandomMissingProbability() const;
-
 	void missingNetworkProbability(double probability);
 	double missingNetworkProbability() const;
 
@@ -97,28 +80,21 @@ public:
 	void initializeMCMCcycle();
 	void MHPstep();
 	int BayesAcceptances(unsigned iteration) const;
-	double sampledBasicRates(unsigned iteration) const;
-	void sampledBasicRates(double value);
-	int sampledBasicRatesDistributions(unsigned iteration) const;
-	void sampledBasicRatesDistributions(int value);
 	double candidates(const EffectInfo * pEffect, unsigned iteration) const;
 	void candidates(const EffectInfo * pEffect, double value);
 
 private:
 	void setStateBefore(MiniStep * pMiniStep);
 	void resetVariables();
+	bool validInsertMissingStep(const Option * pOption,
+		int d0,
+		const MiniStep * pMiniStepA);
+	bool validDeleteMissingStep(MiniStep * pMiniStepA, bool applyTwice);
+	MiniStep * createMiniStep(const Option * pOption,
+		int difference = 0) const;
 
 	Chain * lpChain;
 	bool lsimpleRates;
-	double linsertDiagonalProbability;
-	double lcancelDiagonalProbability;
-	double lpermuteProbability;
-	double linsertPermuteProbability;
-	double ldeletePermuteProbability;
-	double linsertRandomMissingProbability;
-	double ldeleteRandomMissingProbability;
-	double lmissingNetworkProbability;
-	double lmissingBehaviorProbability;
 	double lproposalProbability;
 	bool lmissingData;
 	Aspect laspect;
@@ -126,13 +102,16 @@ private:
 	int lacceptances[7];
 	int lrejections[7];
 	vector<int> lBayesAcceptances;
-	vector<double> lsampledBasicRates;
-	vector<int> lsampledBasicRatesDistributions;
 	map<const EffectInfo *, vector<double> > lcandidates;
+	double lmissingNetworkProbability;
+	double lmissingBehaviorProbability;
 	// current length of permuted interval
 	double lcurrentPermutationLength;
 	int lthisPermutationLength;
+	bool lphase1;
 
+	// A vector of options with missing values in the initial observation
+	vector<const Option *> linitialMissingOptions;
 };
 
 }
